package be.iminds.ilabt.jfed.restapi.debugdata;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.FedmonFullObjectLinkDeserializers;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.JsonLdObjectLinkSerializer;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.lowlevel.connection.ConnectionConfig;
import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.RFC3339Util;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URI;
import java.sql.Timestamp;
import java.util.Map;

/**
 * ClientApiCallDD
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonIgnoreProperties("@context")
public class ClientApiCallDD implements DebugData {
    private static final Logger LOG = LoggerFactory.getLogger(ClientApiCallDD.class);

    private final String api;
    private final String apiVersion;
    private final Timestamp start;
    private final Timestamp stop;
    private final SentData sent;
    private final ReceivedData received;
    private final String stacktrace;

    private final ConnectionDD connection;

    @JsonCreator
    public ClientApiCallDD(@JsonProperty("api") String api,
                           @JsonProperty("apiVersion") String apiVersion,
                           @JsonProperty("start") Timestamp start,
                           @JsonProperty("stop") Timestamp stop,
                           @JsonProperty("sent") SentData sent,
                           @JsonProperty("received") ReceivedData received,
                           @JsonProperty("stacktrace") String stacktrace,
                           @JsonProperty("connection") ConnectionDD connection) {
        this.api = api;
        this.apiVersion = apiVersion;
        this.start = start;
        this.stop = stop;
        this.sent = sent;
        this.received = received;
        this.stacktrace = stacktrace;
        this.connection = connection;
    }

    @JsonProperty
    public String getApi() {
        return api;
    }

    @JsonProperty
    public String getApiVersion() {
        return apiVersion;
    }

    @JsonProperty
    @JsonDeserialize(using = RFC3339Util.TimeStampJsonDeserializer.class)
    @JsonSerialize(using = RFC3339Util.TimeStampJsonSerializer.class)
    public Timestamp getStart() {
        return start;
    }

    @JsonProperty
    @JsonDeserialize(using = RFC3339Util.TimeStampJsonDeserializer.class)
    @JsonSerialize(using = RFC3339Util.TimeStampJsonSerializer.class)
    public Timestamp getStop() {
        return stop;
    }

    @JsonProperty
    public SentData getSent() {
        return sent;
    }

    @JsonProperty
    public ReceivedData getReceived() {
        return received;
    }

    @JsonProperty
    public String getStacktrace() {
        return stacktrace;
    }

    @JsonProperty
    public ConnectionDD getConnection() {
        return connection;
    }

    @JsonProperty(value="@type")
    public String getClassName() {
        return getClass().getSimpleName();
    }

    @JsonProperty(value="@type")
    private void setClassName(String className) {
        if (!getClass().getSimpleName().equals(className))
            throw new IllegalArgumentException("@type must be "+getClass().getSimpleName());
    }


    public static class SentData {
        private final String httpRequestLine;
        private final String httpHeaders;
        private final String httpData;

        private final String xmlRpcMethod;
        private final Object xmlRpcData;

        @JsonCreator
        public SentData(@JsonProperty("httpRequestLine") String httpRequestLine,
                        @JsonProperty("httpHeaders") String httpHeaders,
                        @JsonProperty("httpData") String httpData,
                        @JsonProperty("xmlRpcMethod") String xmlRpcMethod,
                        @JsonProperty("xmlRpcData") Object xmlRpcData) {
            this.httpRequestLine = httpRequestLine;
            this.httpHeaders = httpHeaders;
            this.httpData = httpData;
            this.xmlRpcMethod = xmlRpcMethod;
            this.xmlRpcData = xmlRpcData;
        }

        @JsonProperty
        public String getHttpRequestLine() {
            return httpRequestLine;
        }

        @JsonProperty
        public String getHttpHeaders() {
            return httpHeaders;
        }

        @JsonProperty
        public String getHttpData() {
            return httpData;
        }

        @JsonProperty
        public String getXmlRpcMethod() {
            return xmlRpcMethod;
        }

        @JsonProperty
        public Object getXmlRpcData() {
            return xmlRpcData;
        }
    }

    public static class ReceivedData {
        private final String httpResponseLine;
        private final String httpHeaders;
        private final String httpData;

        private final Object xmlRpcData;

        @JsonCreator
        public ReceivedData(@JsonProperty("httpResponseLine") String httpResponseLine,
                            @JsonProperty("httpHeaders") String httpHeaders,
                            @JsonProperty("httpData") String httpData,
                            @JsonProperty("xmlRpcData") Object xmlRpcData) {
            this.httpResponseLine = httpResponseLine;
            this.httpHeaders = httpHeaders;
            this.httpData = httpData;
            this.xmlRpcData = xmlRpcData;
        }

        @JsonProperty
        public String getHttpResponseLine() {
            return httpResponseLine;
        }

        @JsonProperty
        public String getHttpHeaders() {
            return httpHeaders;
        }

        @JsonProperty
        public String getHttpData() {
            return httpData;
        }

        @JsonProperty
        public Object getXmlRpcData() {
            return xmlRpcData;
        }
    }

    public static class ConnectionDD {
        public ConnectionConfig.Authentication auth;
        public ConnectionConfig.Protocol proto;
        public ConnectionConfig.Type type;
        public URI url;
        public Server server;
        public GeniUrn sslUSer;

        public ConnectionDD(@JsonProperty("auth") ConnectionConfig.Authentication auth,
                            @JsonProperty("proto") ConnectionConfig.Protocol proto,
                            @JsonProperty("type") ConnectionConfig.Type type,
                            @JsonProperty("url") URI url,
                            @JsonProperty("server") Server server,
                            @JsonProperty("sslUSer") GeniUrn sslUSer) {
            this.auth = auth;
            this.proto = proto;
            this.type = type;
            this.url = url;
            this.server = server;
            this.sslUSer = sslUSer;
        }

        public ConnectionDD(@JsonProperty("auth") String auth,
                            @JsonProperty("proto") String proto,
                            @JsonProperty("type") String type,
                            @JsonProperty("url") URI url,
                            @JsonProperty("server") Server server,
                            @JsonProperty("sslUSer") GeniUrn sslUSer) throws IllegalArgumentException{
            this.auth = auth == null ? null : ConnectionConfig.Authentication.valueOf(auth);
            this.proto = proto == null ? null : ConnectionConfig.Protocol.valueOf(proto);;
            this.type = type == null ? null : ConnectionConfig.Type.valueOf(type);;
            this.url = url;
            this.server = server;
            this.sslUSer = sslUSer;
        }

        @JsonProperty
        public ConnectionConfig.Authentication getAuth() {
            return auth;
        }

        @JsonProperty
        public ConnectionConfig.Protocol getProto() {
            return proto;
        }

        @JsonProperty
        public ConnectionConfig.Type getType() {
            return type;
        }

        @JsonProperty
        public URI getUrl() {
            return url;
        }

        @JsonProperty
        @JsonDeserialize(using = FedmonFullObjectLinkDeserializers.ServerDeserializer.class)
        @JsonSerialize(using = JsonLdObjectLinkSerializer.NeverEmbedChildLinkSerializer.class)
        public Server getServer() {
            return server;
        }

        @JsonProperty
        @JsonDeserialize(using = GeniUrn.JsonDeserializer.class)
        @JsonSerialize(using = GeniUrn.JsonSerializer.class)
        public GeniUrn getSslUSer() {
            return sslUSer;
        }
    }

    @Override
    public String toString() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.writerWithDefaultPrettyPrinter().writeValueAsString(this);
        } catch (JsonProcessingException e) {
            LOG.error("Exception converting "+getClass().getSimpleName()+" to JSON", e);
            return "Exception converting "+ getClass().getSimpleName() +" to JSON: "+e.getMessage();
        }
    }
}
